SET SEARCH_PATH = iqp_apprealm;


CREATE TABLE authors
(
    author_id        serial      NOT NULL PRIMARY KEY,
    original_user_id INT         NOT NULL,
    email            VARCHAR(128),
    firstname        VARCHAR(128),
    lastname         VARCHAR(128),
    account_id       INT         NOT NULL,
    username         VARCHAR(45) NOT NULL DEFAULT 'iqp',
    CONSTRAINT uk_author_email UNIQUE (author_id, original_user_id),
    CONSTRAINT fk_account_id FOREIGN KEY (account_id) REFERENCES accounts (account_id)
);
CREATE TABLE installed_packages
(
    package_id   serial             NOT NULL PRIMARY KEY,
    package_uuid VARCHAR(32) UNIQUE NOT NULL,
    package_file TEXT               NOT NULL,
    installed_by INT                NOT NULL,
    installed    TIMESTAMP          NOT NULL
);

CREATE TABLE column_types
(
    type_name    VARCHAR(128) UNIQUE NOT NULL,
    db_name      VARCHAR(128)        NOT NULL,
    sql_type     VARCHAR(128)        NOT NULL,
    allow_unique BOOLEAN             NOT NULL,
    auto_values  BOOLEAN             NOT NULL,
    indexable    BOOLEAN             NOT NULL,
    index_len    SMALLINT            NOT NULL
);


CREATE TYPE PIVOT_TYPES AS ENUM ('METRIC','TYPE','GROUP');

CREATE TABLE entity_tables
(
    table_id               serial,
    table_name             VARCHAR(128) NOT NULL,
    version                INT          NOT NULL,
    grouping_id            INT          NOT NULL,
    created_by             INT                   DEFAULT NULL,
    created                TIMESTAMP    NULL,
    modified_by            INT                   DEFAULT NULL,
    modified               TIMESTAMP    NULL,
    column_data            TEXT         NOT NULL,
    package_id             INT                   DEFAULT NULL,
    protected              BOOLEAN               DEFAULT FALSE,
    is_locked              BOOLEAN      NOT NULL DEFAULT FALSE,
    locked_by              INT          NOT NULL DEFAULT 0,
    is_pivot_table         BOOLEAN      NOT NULL DEFAULT FALSE,
    is_running_pivot_table BOOLEAN      NOT NULL DEFAULT FALSE,
    pivot_table_type       PIVOT_TYPES  NULL     DEFAULT NULL,
    pivot_destination_name VARCHAR(45)  NULL     DEFAULT NULL,
    UUID                   VARCHAR(50)  NOT NULL UNIQUE,
    CONSTRAINT fk_entity_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_entity_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_table_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE entity_relationships
(
    relationship_id serial PRIMARY KEY,
    join_type       varchar(8)  NOT NULL CHECK (join_type IN ('LEFT', 'RIGHT', 'INNER', 'left', 'right', 'inner')),
    multiplicity    varchar(4)  NOT NULL CHECK (multiplicity IN ('1', 'N', 'n')),
    from_table_id   INT         NOT NULL,
    from_table_uuid VARCHAR(45) NOT NULL,
    from_table_ver  INT     DEFAULT NULL,
    from_column_idx INT         NOT NULL,
    to_table_id     INT         NOT NULL,
    to_table_uuid   VARCHAR(45) NOT NULL,
    to_table_ver    INT     DEFAULT NULL,
    to_column_idx   INT         NOT NULL,
    created_by      INT     DEFAULT NULL,
    created         TIMESTAMP   NULL,
    modified_by     INT     DEFAULT NULL,
    modified        TIMESTAMP   NULL,
    package_id      INT     DEFAULT NULL,
    protected       BOOLEAN DEFAULT FALSE,
    UUID            VARCHAR(50) NOT NULL UNIQUE,
    CONSTRAINT uk_relationship_table_ver_column UNIQUE (from_table_uuid, from_table_ver, from_column_idx, to_table_uuid,
                                                        to_table_ver, to_column_idx),
    CONSTRAINT fk_relationship_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_relationship_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_relationship_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE flows
(
    flow_id               serial,
    flow_name             VARCHAR(128) NOT NULL,
    flow_type             VARCHAR(64)  NOT NULL,
    description           TEXT                  DEFAULT NULL,
    version               INT          NOT NULL,
    grouping_id           INT          NOT NULL,
    created_by            INT                   DEFAULT NULL,
    created               TIMESTAMP    NULL,
    modified_by           INT                   DEFAULT NULL,
    modified              TIMESTAMP    NULL,
    impl_details          TEXT         NOT NULL,
    drools_uuid           TEXT                  DEFAULT NULL,
    drools_version        INT                   DEFAULT NULL,
    drools_wid            VARCHAR(4),
    wid                   INT                   DEFAULT NULL,
    details               TEXT         NOT NULL,
    inputs                TEXT         NOT NULL,
    outputs               TEXT         NOT NULL,
    detailed_inputs       TEXT         NOT NULL,
    detailed_outputs      TEXT         NOT NULL,
    package_id            INT                   DEFAULT NULL,
    protected             BOOLEAN               DEFAULT FALSE,
    is_locked             BOOLEAN      NOT NULL DEFAULT FALSE,
    locked_by             INT          NOT NULL DEFAULT 0,
    UUID                  VARCHAR(50)  NOT NULL UNIQUE,
    request               text         null,
    request_datasource_id int          null,
    ignored_nodes         text         null,
    CONSTRAINT fk_flow_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_flow_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_flow_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE queries
(
    query_id    serial,
    version     INT          NOT NULL,
    query_name  VARCHAR(128) NOT NULL,
    description TEXT    DEFAULT NULL,
    grouping_id INT          NOT NULL,
    created_by  INT     DEFAULT NULL,
    created     TIMESTAMP    NULL,
    wid         INT     DEFAULT NULL,
    contents    TEXT         NOT NULL,
    inputs      TEXT         NOT NULL,
    outputs     TEXT         NOT NULL,
    package_id  INT     DEFAULT NULL,
    protected   BOOLEAN DEFAULT FALSE,
    CONSTRAINT pk_query_id PRIMARY KEY (query_id, version),
    CONSTRAINT fk_query_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_query_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE query_histories
(
    query_id        INT       NOT NULL,
    current_version INT       NOT NULL,
    prev_version    INT,
    modified_by     INT  DEFAULT NULL,
    modified        TIMESTAMP NULL,
    grouping_id     INT       NOT NULL,
    contents        TEXT      NOT NULL,
    name            TEXT      NOT NULL,
    description     TEXT DEFAULT NULL,
    CONSTRAINT fk_history_query_id PRIMARY KEY (query_id, current_version),
    CONSTRAINT fk_query_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_query_prev_version FOREIGN KEY (query_id, prev_version) REFERENCES query_histories (query_id, current_version)
);

CREATE TABLE entity_histories
(
    table_id        INT       NOT NULL,
    current_version INT       NOT NULL,
    prev_version    INT,
    modified_by     INT  DEFAULT NULL,
    modified        TIMESTAMP NULL,
    grouping_id     INT       NOT NULL,
    column_data     TEXT      NOT NULL,
    name            TEXT      NOT NULL,
    description     TEXT DEFAULT NULL,
    CONSTRAINT fk_entity_history_table_id PRIMARY KEY (table_id, current_version),
    CONSTRAINT fk_entity_history_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_predecessor_entity FOREIGN KEY (table_id, prev_version) REFERENCES entity_histories (table_id, current_version)
);

CREATE TABLE sub_queries
(
    sub_query_id      INT NOT NULL,
    sub_query_version INT NOT NULL,
    query_id          INT NOT NULL,
    query_version     INT NOT NULL
);

CREATE TABLE query_tables
(
    table_id      INT NOT NULL,
    table_version INT NOT NULL,
    query_id      INT NOT NULL,
    query_version INT NOT NULL
);

CREATE TABLE rules
(
    rule_id       serial,
    version       INT          NOT NULL,
    rule_name     VARCHAR(128) NOT NULL,
    description   TEXT    DEFAULT NULL,
    grouping_id   INT          NOT NULL,
    created_by    INT     DEFAULT NULL,
    created       TIMESTAMP    NULL,
    modified_by   INT     DEFAULT NULL,
    modified      TIMESTAMP    NULL,
    wid           INT     DEFAULT NULL,
    bp_uuid       TEXT    DEFAULT NULL,
    bp_version    INT     DEFAULT NULL,
    bp_definition TEXT         NOT NULL,
    inputs        TEXT         NOT NULL,
    outputs       TEXT         NOT NULL,
    user_actions  TEXT         NOT NULL,
    events        TEXT         NOT NULL,
    package_id    INT     DEFAULT NULL,
    protected     BOOLEAN DEFAULT FALSE,
    CONSTRAINT pk_rule_id PRIMARY KEY (rule_id, version),
    CONSTRAINT fk_rule_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_rule_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_rule_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE functions
(
    function_id      serial PRIMARY KEY,
    function_name    VARCHAR(128) NOT NULL,
    version          INT          NOT NULL,
    description      TEXT    DEFAULT NULL,
    grouping_id      INT          NOT NULL,
    created_by       INT     DEFAULT NULL,
    created          TIMESTAMP    NULL,
    modified_by      INT     DEFAULT NULL,
    modified         TIMESTAMP    NULL,
    wid              INT     DEFAULT NULL,
    class_name       VARCHAR(127) NOT NULL,
    method_name      VARCHAR(127) NOT NULL,
    constructor      TEXT         NOT NULL,
    inputs           TEXT         NOT NULL,
    outputs          TEXT         NOT NULL,
    detailed_inputs  TEXT         NOT NULL,
    detailed_outputs TEXT         NOT NULL,
    package_id       INT     DEFAULT NULL,
    protected        BOOLEAN DEFAULT FALSE,
    CONSTRAINT uk_function UNIQUE (function_name, version),
    CONSTRAINT fk_function_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);


CREATE TABLE apps
(
    app_id         serial PRIMARY KEY,
    app_name       VARCHAR(128) NOT NULL,
    description    TEXT                  DEFAULT NULL,
    version        INT          NOT NULL,
    grouping_id    INT          NOT NULL,
    created_by     INT                   DEFAULT NULL,
    created        TIMESTAMP    NULL,
    modified_by    INT                   DEFAULT NULL,
    modified       TIMESTAMP    NULL,
    theme_id       INT                   DEFAULT NULL,
    theme_uuid     VARCHAR(50)           DEFAULT NULL,
    items          TEXT         NOT NULL,
    globals        TEXT                  DEFAULT NULL,
    icon_url       TEXT                  DEFAULT NULL,
    logo_url       TEXT                  DEFAULT NULL,
    is_locked      BOOLEAN      NOT NULL DEFAULT FALSE,
    locked_by      INT          NOT NULL DEFAULT 0,
    UUID           VARCHAR(50)  NOT NULL UNIQUE,
    app_descriptor VARCHAR(450) NULL     DEFAULT NULL,
    was_imported   BOOLEAN               DEFAULT FALSE,
    CONSTRAINT fk_app_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_app_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id)
);


CREATE TABLE plugin_management
(
    pm_id           serial PRIMARY KEY,
    plugin_typename VARCHAR(128),
    app_id          INT NOT NULL,
    CONSTRAINT fk_pm_app_id FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE pages
(
    page_id       serial PRIMARY KEY,
    app_id        INT          NOT NULL,
    page_name     VARCHAR(128) NOT NULL,
    description   TEXT DEFAULT NULL,
    version       INT          NOT NULL,
    grouping_id   INT          NOT NULL,
    created_by    INT  DEFAULT NULL,
    created       TIMESTAMP    NULL,
    modified_by   INT  DEFAULT NULL,
    modified      TIMESTAMP    NULL,
    allComponents TEXT         NULL,
    components    TEXT         NOT NULL,
    flows         TEXT DEFAULT NULL,
    tables        TEXT DEFAULT NULL,
    UUID          VARCHAR(50)  NOT NULL UNIQUE,
    app_uuid      VARCHAR(50)  NOT NULL,
    page_icon     TEXT         NULL,
    CONSTRAINT fk_page_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_page_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_page_app FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE themes
(
    theme_id      serial PRIMARY KEY,
    theme_name    VARCHAR(128) NOT NULL,
    description   TEXT              DEFAULT NULL,
    version       INT          NOT NULL,
    grouping_id   INT          NOT NULL,
    created_by    INT               DEFAULT NULL,
    created       TIMESTAMP    NULL,
    modified_by   INT               DEFAULT NULL,
    modified      TIMESTAMP    NULL,
    settings      TEXT         NOT NULL,
    default_theme BOOLEAN      NOT NULL,
    package_id    INT               DEFAULT NULL,
    UUID          VARCHAR(50)  NOT NULL UNIQUE,
    updated       BOOLEAN      NULL DEFAULT FALSE,
    protected     BOOLEAN      NULL DEFAULT FALSE,
    CONSTRAINT fk_theme_creator FOREIGN KEY (created_by) REFERENCES authors (author_id),
    CONSTRAINT fk_theme_modifier FOREIGN KEY (modified_by) REFERENCES authors (author_id),
    CONSTRAINT fk_theme_package_id FOREIGN KEY (package_id) REFERENCES installed_packages (package_id)
);

CREATE TABLE wid_outputs
(
    id          serial PRIMARY KEY,
    instance_id BIGINT    NOT NULL,
    wid         BIGINT    NOT NULL,
    output      TEXT      NOT NULL,
    created     TIMESTAMP NOT NULL,
    updated     TIMESTAMP NOT NULL,
    user_id     INT DEFAULT NULL
);

CREATE TABLE last_wid
(
    id       serial PRIMARY KEY,
    prefix   VARCHAR(128) DEFAULT 'IQP_WID_',
    wid      BIGINT NOT NULL,
    modified TIMESTAMP
);

CREATE TABLE groupings
(
    grouping_id        serial PRIMARY KEY,
    parent_grouping_id INT,
    grouping_name      VARCHAR(128) UNIQUE NOT NULL,
    description        TEXT,
    created_by         INT                 NOT NULL,
    created            TIMESTAMP           NOT NULL,
    modified_by        INT                 NOT NULL,
    modified           TIMESTAMP           NOT NULL,
    CONSTRAINT fk_groupings_self FOREIGN KEY (parent_grouping_id) REFERENCES groupings (grouping_id)
);

CREATE TABLE app_feature_tags
(
    feature_tag  VARCHAR(64) UNIQUE PRIMARY KEY,
    feature_name VARCHAR(128),
    description  TEXT
);

CREATE TABLE templates
(
    template_id serial              NOT NULL PRIMARY KEY,
    name        VARCHAR(128) UNIQUE NOT NULL,
    description TEXT DEFAULT NULL,
    version     INT                 NOT NULL,
    created     TIMESTAMP           NOT NULL,
    created_by  INT                 NOT NULL,
    modified    TIMESTAMP           NOT NULL,
    modified_by INT                 NOT NULL,
    image_url   TEXT DEFAULT NULL
);

CREATE TABLE template_apps
(
    template_id INT NOT NULL,
    app_id      INT NOT NULL
);

CREATE TABLE template_regions
(
    template_id INT        NOT NULL,
    region      VARCHAR(4) NOT NULL
);

CREATE TABLE template_languages
(
    template_id INT        NOT NULL,
    language    VARCHAR(4) NOT NULL
);


CREATE TYPE BILLING_TYPE as ENUM ('trial','once','daily','monthly','yearly');
CREATE TYPE CURRENCY as ENUM ('JPY','USD','EUR');

CREATE TABLE template_billing_options
(
    template_id  INT          NOT NULL,
    billing_type BILLING_TYPE NOT NULL,
    available    BOOLEAN      NOT NULL,
    period_days  INT      DEFAULT NULL,
    amount       INT      DEFAULT NULL,
    currency     CURRENCY DEFAULT NULL
);

CREATE TABLE template_features
(
    template_id  INT     NOT NULL,
    feature_tag  VARCHAR(64),
    feature_flag BOOLEAN NOT NULL,
    CONSTRAINT fk_app_template_features_tag FOREIGN KEY (feature_tag) REFERENCES app_feature_tags (feature_tag)
);

CREATE TABLE app_billing_options
(
    app_id       INT          NOT NULL,
    billing_type BILLING_TYPE NOT NULL,
    available    BOOLEAN      NOT NULL,
    period_days  INT      DEFAULT NULL,
    amount       INT      DEFAULT NULL,
    currency     CURRENCY DEFAULT NULL,
    CONSTRAINT fk_app_billing_options_app FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE app_regions
(
    app_id INT        NOT NULL,
    region VARCHAR(4) NOT NULL,
    CONSTRAINT fk_app_regions_app FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE app_languages
(
    app_id   INT        NOT NULL,
    language VARCHAR(4) NOT NULL,
    CONSTRAINT fk_app_languages_app FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE app_features
(
    app_id       INT     NOT NULL,
    feature_tag  VARCHAR(64),
    feature_flag BOOLEAN NOT NULL,
    CONSTRAINT fk_app_features_tag FOREIGN KEY (feature_tag) REFERENCES app_feature_tags (feature_tag),
    CONSTRAINT fk_app_features_app FOREIGN KEY (app_id) REFERENCES apps (app_id)
);

CREATE TABLE wid_transactions
(
    id             serial             NOT NULL PRIMARY KEY,
    transaction_id VARCHAR(32) UNIQUE NOT NULL,
    user_id        INT                NOT NULL
);

CREATE TABLE image_resources
(
    file_path     VARCHAR(191),
    file_url      TEXT,
    owner_user_id INT,
    ref_count     BIGINT,
    PRIMARY KEY (file_path)
);

CREATE TABLE app_logs
(
    ID         serial      NOT NULL PRIMARY KEY,
    APP        VARCHAR(20) NOT NULL,
    USER_ID    VARCHAR(20) NOT NULL,
    ACCOUNT_ID VARCHAR(20) NOT NULL,
    DATED      TIMESTAMP   NOT NULL,
    LOGGER     TEXT        NOT NULL,
    LEVEL      VARCHAR(10) NOT NULL,
    MESSAGE    TEXT DEFAULT NULL

);

CREATE TABLE server_variables_metadata
(
    int_id                serial      NOT NULL,
    metric_parameter_name VARCHAR(45) NOT NULL,
    parameter_type        VARCHAR(45) NOT NULL,
    PRIMARY KEY (int_id)
);

CREATE TABLE server_variables
(
    id             serial       NOT NULL,
    parameter_name VARCHAR(450) NOT NULL,
    data_source    VARCHAR(45)  NULL,
    query_id       INT          NULL,
    query_uuid     VARCHAR(45)  NULL,
    inputs         VARCHAR(400) NULL,
    app_id         INT          NULL,
    account_id     INT          NOT NULL,
    fixed_value    VARCHAR(45)  NULL,
    modified       TIMESTAMP    NULL,
    modified_by    INT          NULL,
    UUID           VARCHAR(50)  NOT NULL UNIQUE,
    PRIMARY KEY (id)
);

CREATE TABLE saved_images
(
    image_id      serial        NOT NULL,
    image_url     VARCHAR(450)  NOT NULL,
    original_name VARCHAR(100)  NULL,
    current_name  VARCHAR(1000) NOT NULL,
    description   VARCHAR(300)  NULL,
    account_id    INT           NOT NULL,
    user_id       INT           NULL,
    local_url     VARCHAR(300)  NOT NULL,
    PRIMARY KEY (image_id)
);

CREATE TABLE emails_servers
(
    id                   serial       NOT NULL,
    server_logical_name  VARCHAR(150) NOT NULL,
    smtp_server_adress   VARCHAR(300) NULL,
    smtp_port            VARCHAR(100) NOT NULL,
    smtp_server_password VARCHAR(100) NOT NULL,
    mail_default_from    VARCHAR(100) NOT NULL,
    account_id           INT          NOT NULL,
    PRIMARY KEY (id)
);

CREATE TABLE installed_patches
(
    id             serial      NOT NULL,
    patch_version  VARCHAR(45) NOT NULL,
    server_patch   BOOLEAN     NOT NULL,
    client_patch   TEXT        NOT NULL,
    installed_date TIMESTAMP   NOT NULL DEFAULT CURRENT_TIMESTAMP,
    status         VARCHAR(45) NULL,
    PRIMARY KEY (id)
);

CREATE TABLE installed_versions
(
    id                      serial       NOT NULL,
    server_version          VARCHAR(45)  NOT NULL,
    server_version_uploaded TIMESTAMP    NOT NULL DEFAULT CURRENT_TIMESTAMP,
    client_version          VARCHAR(45)  NOT NULL,
    client_version_uploaded TIMESTAMP    NOT NULL,
    versions_available      VARCHAR(191) NULL,
    PRIMARY KEY (id)
);

CREATE TABLE server_settings
(
    id       serial       NOT NULL,
    setting  VARCHAR(191) NOT NULL,
    value    VARCHAR(191) NOT NULL,
    account  INT          NOT NULL,
    grouping VARCHAR(191) NOT NULL,
    PRIMARY KEY (id, setting, account)
);

CREATE TABLE app_history
(
    commit_id           serial       NOT NULL,
    app_id              INT          NOT NULL,
    author_id           INT          NOT NULL,
    account_id          INT          NOT NULL,
    first_name          VARCHAR(45)  NULL,
    last_name           VARCHAR(45)  NULL,
    username            VARCHAR(45)  NULL,
    comment             VARCHAR(500) NULL,
    timestamp           TIMESTAMP    NULL,
    app_component_saved VARCHAR(500) NULL,
    PRIMARY KEY (commit_id)
);

CREATE TABLE site_app_migrate_users
(
    id                       serial       NOT NULL,
    site_username            varchar(127) NOT NULL,
    app_username             varchar(127) NOT NULL,
    site_password            varchar(127)          DEFAULT NULL,
    app_password             varchar(127)          DEFAULT NULL,
    account_id               int          NOT NULL,
    password_expiration_date TIMESTAMP    NOT NULL DEFAULT '1970-01-02 02:02:02',
    app_first_name           VARCHAR(450) NULL,
    app_last_name            VARCHAR(450) NULL,
    PRIMARY KEY (id)
);

CREATE TABLE account_settings
(
    id       serial       NOT NULL,
    account  INT          NOT NULL,
    setting  VARCHAR(191) NOT NULL,
    value    VARCHAR(191) NOT NULL,
    grouping VARCHAR(191) NOT NULL,
    PRIMARY KEY (id, account)
);

CREATE TABLE failed_logins
(
    user_name VARCHAR(50) NOT NULL,
    count     INT         NULL DEFAULT 0,
    PRIMARY KEY (user_name)
);

CREATE TABLE previous_passwords
(
    username  VARCHAR(100) NOT NULL,
    passwords VARCHAR(850) NULL,
    PRIMARY KEY (username)
);

CREATE TABLE roles
(
    role_uuid   VARCHAR(50)  NOT NULL,
    role_name   VARCHAR(127) NOT NULL,
    account_id  INT          NOT NULL,
    created_by  INT          NOT NULL,
    created     TIMESTAMP    NULL,
    modified_by INT          NOT NULL,
    modified    TIMESTAMP    NULL,
    parent_uuid VARCHAR(127) NULL,
    parent_name VARCHAR(127) NULL,
    root_uuid   VARCHAR(127) NULL,
    root_name   VARCHAR(127) NULL,
    CONSTRAINT uk_role_uuid UNIQUE (role_uuid)
);

CREATE TABLE roles_users
(
    role_uuid VARCHAR(50) NOT NULL,
    user_id   INT,
    CONSTRAINT fk_roles_users_role_uuid FOREIGN KEY (role_uuid) REFERENCES roles (role_uuid)
);


CREATE TABLE role_path
(
    role_uuid VARCHAR(50)  NOT NULL,
    role_name VARCHAR(127) NOT NULL,
    path_uuid VARCHAR(50)  NOT NULL,
    path_name VARCHAR(127) NOT NULL,
    CONSTRAINT fk_role_path_role_uuid FOREIGN KEY (role_uuid) REFERENCES roles (role_uuid)
);


CREATE TABLE roles_management
(
    user_id    INT,
    username   VARCHAR(127) NOT NULL,
    account_id INT          NOT NULL
);


CREATE TYPE QUERY_TYPE as ENUM ('SELECT','INSERT','UPDATE');
CREATE TYPE ENTITY_TYPE as ENUM ('METRIC','TYPE','GROUP');

CREATE TABLE m2m_pivot_queries
(
    uuid                   VARCHAR(55)  NOT NULL,
    query_sql              VARCHAR(500) NULL,
    query_type             QUERY_TYPE   NULL,
    entity_type            ENTITY_TYPE  NULL,
    query_destination_name VARCHAR(45)  NULL DEFAULT NULL,
    pivot_table_uuid       VARCHAR(45)  NULL DEFAULT NULL,
    PRIMARY KEY (uuid)
);

CREATE TABLE encoded_images
(
    id         serial      NOT NULL,
    image      TEXT        NULL,
    account_id INT         NULL,
    user_id    INT         NULL,
    image_name VARCHAR(45) NULL,
    PRIMARY KEY (id)
);

CREATE TABLE main_widget
(
    id            serial       NOT NULL,
    account_id    INT          NOT NULL,
    app_id        VARCHAR(100) NULL,
    page_id       VARCHAR(100) NULL,
    main_image_id INT          NULL,
    PRIMARY KEY (id)
);


CREATE TABLE small_widget
(
    id             serial NOT NULL,
    account_id     INT    NOT NULL,
    main_widget_id INT    NOT NULL,
    image_id       INT    NOT NULL,
    PRIMARY KEY (id, image_id)
);

CREATE TABLE _usage
(
    id            serial       NOT NULL,
    hour_datetime TIMESTAMP    NOT NULL,
    type          varchar(45)  NOT NULL,
    _key          varchar(128) NOT NULL,
    count         int          DEFAULT NULL,
    size          bigint       DEFAULT NULL,
    general_str   varchar(256) DEFAULT NULL,
    general_int   int          DEFAULT NULL,
    PRIMARY KEY (hour_datetime, type, _key)
);


CREATE VIEW view_usage_recent AS
select _usage.id            AS id,
       _usage.hour_datetime AS hour_datetime,
       _usage.type          AS type,
       _usage._key          AS _key,
       _usage.count         AS count,
       _usage.size          AS size,
       _usage.general_str   AS general_str,
       _usage.general_int   AS general_int
from _usage
where (_usage.hour_datetime = (select max(_usage.hour_datetime) from _usage));

CREATE VIEW view_usage_sum AS
select _usage.type AS type, sum(_usage.count) AS count, sum(_usage.size) AS size
from _usage
group by _usage.type;

CREATE VIEW view_usage_sum_recent AS
select view_usage_recent.type AS type, sum(view_usage_recent.count) AS count, sum(view_usage_recent.size) AS size
from view_usage_recent
group by view_usage_recent.type;

CREATE VIEW view_usage_top_apps AS
select _usage._key AS _key, floor((sum(_usage.count) / 2)) AS count, sum(_usage.size) AS size
from _usage
where ((_usage.type like 'request_%') and (_usage._key like '%app_id=%'))
group by _usage._key
order by size desc
limit 3;

CREATE VIEW view_usage_top_apps_recent AS
select view_usage_recent._key                    AS _key,
       floor((sum(view_usage_recent.count) / 2)) AS count,
       sum(view_usage_recent.size)               AS size
from view_usage_recent
where ((view_usage_recent.type like 'request_%') and (view_usage_recent._key like '%app_id=%'))
group by view_usage_recent._key
order by size desc
limit 3;

CREATE VIEW view_usage_top_events AS
select _usage._key AS _key, sum(_usage.count) AS count, sum(_usage.size) AS size
from _usage
where (_usage.type = 'event')
group by _usage._key
order by size desc
limit 3;

CREATE VIEW view_usage_top_events_recent AS
select view_usage_recent._key AS _key, sum(view_usage_recent.count) AS count, sum(view_usage_recent.size) AS size
from view_usage_recent
where (view_usage_recent.type = 'event')
group by view_usage_recent._key
order by size desc
limit 3;


